@@ -1,18 +1,28 @@
-/* $Id: DBIXS.h,v 1.48 1998/08/09 20:48:38 timbo Exp $
+/* $Id: DBIXS.h,v 10.8 1999/05/09 01:14:04 timbo Exp $
  *
- * Copyright (c) 1994, 1995, 1996, 1997  Tim Bunce  England
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999  Tim Bunce  England
  *
  * See COPYRIGHT section in DBI.pm for usage and distribution rights.
  */
 
 /* DBI Interface Definitions for DBD Modules */
 
+#ifndef DBIXS_VERSION				/* prevent multiple inclusion */
 
-/* first pull in the standard Perl header files for extensions */
+#ifndef DBIS
+#define DBIS	dbis	/* default name for dbistate_t variable	*/
+#endif
+
+/* first pull in the standard Perl header files for extensions	*/
+#define PERL_POLLUTE
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
 
+#ifdef debug		/* causes problems with DBIS->debug	*/
+#undef debug
+#endif
+
 /* Perl backwards compatibility definitions */
 #ifndef boolSV	/* added in perl5.004		*/
 #define boolSV(bool) ((bool) ? &sv_yes : &sv_no) 
@@ -46,11 +56,17 @@ error You_need_to_upgrade_your_DBI_module_before_building_this_driver
 #endif
 
 
-#ifdef USE_THREADS
-/* #define DBI_USE_THREADS DOES NOT WORK - recursive calls hang */
+#if defined(USE_THREADS) && !defined(DBI_NO_THREADS)
+#define DBI_USE_THREADS
+#define DBI_LOCK	MUTEX_LOCK(DBIS->mutex)
+#define DBI_UNLOCK	MUTEX_UNLOCK(DBIS->mutex)
 #define dbi_mutex perl_mutex
+#define dbi_cond  perl_cond
 #else
+#define DBI_LOCK
+#define DBI_UNLOCK
 #define dbi_mutex void
+#define dbi_cond  void
 #endif
 
 
@@ -79,17 +95,17 @@ typedef struct dbih_com_std_st {
     U32  flags;
     int  call_depth;	/* used by DBI to track nested calls (int)	*/
     U16  type;		/* DBIt_DR, DBIt_DB, DBIt_ST			*/
-    SV   *my_h_obj;	/* copy of own SvRV(inner handle) (NO r.c.inc)	*/
+    SV   *my_h;		/* copy of outer handle HV (not refcounted)	*/
     SV   *parent_h;	/* parent inner handle (RV(HV)) (r.c.inc)	*/
     imp_xxh_t *parent_com;	/* parent com struct shortcut		*/
-    dbi_mutex *h_mutex;	/* concept placeholder for (driver?) mutex	*/
+    dbi_cond  *thr_cond;/* condition for thread access (see dispatch)	*/
 
     HV   *imp_stash;	/* who is the implementor for this handle	*/
     SV   *imp_data;	/* optional implementors data (for perl imp's)	*/
 
     I32  kids;		/* count of db's for dr's, st's for db's etc	*/
     I32  active_kids;	/* kids which are currently DBIc_ACTIVE		*/
-    I32  spare1;
+    U32  thr_user;	/* thread id currently using the handle		*/
     void *spare2;
 } dbih_com_std_t;
 
@@ -145,6 +161,8 @@ typedef struct {		/* -- STATEMENT --			*/
     void *spare2;
 } dbih_stc_t;
 
+
+/* XXX THIS STRUCTURE SHOULD NOT BE USED */
 typedef struct {		/* -- FIELD DESCRIPTOR --		*/
     dbih_com_std_t	std;	/* standard structure (not fully setup)	*/
 
@@ -170,10 +188,12 @@ typedef struct {		/* -- FIELD DESCRIPTOR --		*/
 #define DBIc_FLAGS(imp)		_imp2com(imp, std.flags)
 #define DBIc_TYPE(imp)		_imp2com(imp, std.type)
 #define DBIc_CALL_DEPTH(imp)	_imp2com(imp, std.call_depth)
-#define DBIc_MY_H_OBJ(imp)  	_imp2com(imp, std.my_h_obj)
+#define DBIc_MY_H(imp)  	_imp2com(imp, std.my_h)
 #define DBIc_PARENT_H(imp)  	_imp2com(imp, std.parent_h)
 #define DBIc_PARENT_COM(imp)  	_imp2com(imp, std.parent_com)
-#define DBIc_MUTEX(imp)  	_imp2com(imp, std.h_mutex)
+#define DBIc_THR_COND(imp)  	_imp2com(imp, std.thr_cond)
+#define DBIc_THR_USER(imp)  	_imp2com(imp, std.thr_user)
+#define DBIc_THR_USER_NONE  	(~(unsigned long)0)
 #define DBIc_IMP_STASH(imp)  	_imp2com(imp, std.imp_stash)
 #define DBIc_IMP_DATA(imp)  	_imp2com(imp, std.imp_data)
 #define DBIc_KIDS(imp)  	_imp2com(imp, std.kids)
@@ -194,6 +214,7 @@ typedef struct {		/* -- FIELD DESCRIPTOR --		*/
 /*	sth	*/
 #define DBIc_NUM_FIELDS(imp)  	_imp2com(imp, num_fields)
 #define DBIc_NUM_PARAMS(imp)  	_imp2com(imp, num_params)
+#define DBIc_NUM_PARAMS_AT_EXECUTE	-9 /* see Driver.xst */
 #define DBIc_ROW_COUNT(imp)  	_imp2com(imp, row_count)
 #define DBIc_FIELDS_AV(imp)  	_imp2com(imp, fields_svav)
 #define DBIc_FDESC_AV(imp)  	_imp2com(imp, fields_fdav)
@@ -212,13 +233,14 @@ typedef struct {		/* -- FIELD DESCRIPTOR --		*/
 #define DBIcf_AutoCommit  0x0200	/* dbh only. used by drivers		*/
 #define DBIcf_LongTruncOk 0x0400	/* truncation to LongReadLen is okay	*/
 #define DBIcf_MultiThread 0x0800	/* allow multiple threads to enter	*/
+#define DBIcf_Taint       0x1000	/* taint fetched data			*/
 
 #define DBIcf_INHERITMASK			/* what NOT to pass on to children */	\
 	(U32)( DBIcf_COMSET | DBIcf_IMPSET | DBIcf_ACTIVE | DBIcf_IADESTROY		\
 	/* These are for dbh only:	*/						\
 	| DBIcf_AutoCommit	)
 
-/* general purpose flag setting and testing macros */
+/* general purpose flag setting and testing macros (except ACTIVE)	*/
 #define DBIc_is(imp, flag)	(DBIc_FLAGS(imp) &   (flag))
 #define DBIc_has(imp,flag)	DBIc_is(imp, flag) /* alias for _is */
 #define DBIc_on(imp, flag)	(DBIc_FLAGS(imp) |=  (flag))
@@ -307,6 +329,9 @@ typedef struct {		/* -- FIELD DESCRIPTOR --		*/
 #define DBEVENT_event	"DBEVENT"
 #define UNKNOWN_event	"UNKNOWN"
 
+
+/* --- Handy Macros --- */
+
 #define DBIh_CLEAR_ERROR(imp_xxh) (void)( \
 	(void)SvOK_off(DBIc_ERR(imp_xxh)),    	\
 	(void)SvOK_off(DBIc_ERRSTR(imp_xxh)),	\
@@ -358,9 +383,6 @@ typedef struct {
 #define set_attr(h, k, v)	set_attr_k(h, k, 0, v)
 #define get_attr(h, k)		get_attr_k(h, k, 0)
 
-#ifndef DBIS
-#define DBIS              dbis /* default name for dbistate_t variable	*/
-#endif
 #define DBISTATE_DECLARE  static dbistate_t *DBIS
 #define DBISTATE_PERLNAME "DBI::_dbistate"
 #define DBISTATE_ADDRSV   (perl_get_sv(DBISTATE_PERLNAME, 0x05))
@@ -406,4 +428,5 @@ typedef struct {
 	if ((svp=DBD_ATTRIB_GET_SVP(attribs, key,klen)) != NULL)	\
 	    var = SvIV(*svp)
 
+#endif /* DBIXS_VERSION */
 /* end of DBIXS.h */
